
//////////////////////////////////////////////////////////////////////////////
//
//  socket.C
//
//  $Log$
//
//	Written by Shaun Terry, Copyright 1995
//
//////////////////////////////////////////////////////////////////////////////

#ifndef lint
static const char rcsid[] = "$Id$";
#endif

#include <sys/stat.h>
#include <sys/uio.h>
#include <poll.h>
#include <ctype.h>
#include "socket.h"

extern "C" {
	int close(int);
	int shutdown(int, int);
	int write(int, char *, int);
	int read(int, char *, int);
	int gethostname(char *, int);
	int getdtablesize();
}

Socket::Socket()
:sock(InvalidSocket)
{
}

int Socket::openHost(char *host, char *serviceName)
{
	servent *service = getservbyname(serviceName, "tcp");
	return service == NULL ? -1 : openHost(host, service->s_port);
}

Socket::~Socket()
{
    if (sock != InvalidSocket)
		::close(sock);
}

// 
// Make a connection to a remote process
// using the host name and the port number
//
// Note: the host name must be in the 
// local hosts file, or in the yp hosts file.
//
//
int Socket::openHost( char *host, int port )
{
    struct hostent *hostEntry;
    struct sockaddr_in internetAddr;
    struct stat buf;

    int err = fstat(sock, &buf);

    if (err < 0 || sock == InvalidSocket)
	{
		close();

		sock = socket(AF_INET, SOCK_STREAM, 0);
		if (sock < 0)
		{
			perror("socket");
			sock = InvalidSocket;
			return(-1);
		}
	}
//  else re-use the same socket

    bzero(&internetAddr, sizeof(internetAddr));
	if (isdigit(*host))
	{
		internetAddr.sin_addr.s_addr = inet_addr(host);
		internetAddr.sin_family = AF_INET;		// good guess, no?
	}
	else
	{
		if ((hostEntry = gethostbyname(host)) == (struct hostent *)0)
		{
			fprintf(stderr, "gethostbyname(%s) failed\n", host);
			return(-1);
		}
		bcopy((char *)hostEntry->h_addr, 
			  (char *)&internetAddr.sin_addr, 
			  hostEntry->h_length);
		internetAddr.sin_family = hostEntry->h_addrtype;
	}


    internetAddr.sin_port   = htons(port);
    if ( connect(sock, (sockaddr *)&internetAddr, sizeof(internetAddr)) < 0) {
        close();
        return(-1);
    }

    portNumber = getPortOfSocket(sock);

    return(0);
}

// 
// Normal close				
//
int Socket::close()
{
    if ( sock == InvalidSocket ) { return(-1); }

    ::close(sock);
    sock = InvalidSocket;

    return(0); 
}

// 
// Nice close				
// Lets sender drain
//
int Socket::niceClose()
{
    if ( sock == InvalidSocket ) { return(-1); }

    return( shutdown(sock, 1) );
}

// 
// Send some data
//
int Socket::send( char *message, int length )
{
    if ( sock == InvalidSocket ) { return(-1); }

//    return( ::send(sock, (char *) message, length, 0 ) );
    return( write(sock, message, length ) );
}

// 
// Read some data
//
int Socket::recv(char *pBuf, int length)
{
    if ( sock == InvalidSocket ) { return(-1); }

::recv(sock, pBuf, length, 0);

/*
	register int bytesSoFar = 0;
	int tries=0;
	
	while (bytesSoFar < length)
	{
		int n = read(sock, pBuf+bytesSoFar, length-bytesSoFar);
		// If call interrupted try again a few times.
		if (n == -1 && errno == EINTR) {
			if (++tries == 5) return -1;
		}
		else if (n == -1) return -1;
		else if (n == 0) return 0;
			
		bytesSoFar += n;
	}

	return bytesSoFar;
*/
return length;
}

int Socket::hostListen( int portNum, int maxConns )
{
    int on = 1;
    char   host[30];
    struct hostent *hostEntry;
    struct sockaddr_in  inetAddr;

	portNumber = portNum;

    //
    // Acquire a socket in the Internet Domain
    //

    if ((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		return(-1);
    }

    if ( setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
		(char *)&on, sizeof(on)) < 0){
		return(-1);
    }

    //
    // Initialize "sockaddr_in" structure for bind
    //
    if (gethostname(host, sizeof(host)) < 0) { return(-1); }

    if ((hostEntry = gethostbyname(host)) == (struct hostent *)0) {
		return(-1);
    }

    //
    // Listen on "PORT" for connection requests from any client
    //
    bzero(&inetAddr, sizeof(inetAddr));
    inetAddr.sin_family      = hostEntry->h_addrtype;
    inetAddr.sin_addr.s_addr = INADDR_ANY;
    inetAddr.sin_port        = portNum;

    //
    // bind local address to socket
    //
    if (bind(sock, (sockaddr *)&inetAddr, sizeof(inetAddr)) < 0) {
		return(-1);
	}

    //
    // listen
    //
    if (listen(sock, maxConns) < 0) { return(-1); }

    if (portNumber == 0) portNumber = getPortOfSocket(sock);

    return(sock);
}

int Socket::hostListen(char *serviceName, int maxConnections)
{
	// Look for the service using symbolic name. Since this is a
	// reliable socket, we are only interested in the TCP port number.
	//
	servent *service = getservbyname(serviceName, "tcp");
	if (service == NULL)
		return -1;
	else
		return hostListen(service->s_port, maxConnections);
}

int Socket::getPortOfSocket(int sockId)
{
    struct sockaddr_in saddr;
    int len;

    len = sizeof(saddr);
    if (getsockname(sockId, (struct sockaddr *) &saddr, &len) < 0) {
	return -1;
    }

    return saddr.sin_port;
}

int Socket::accept(struct sockaddr_in *from, int *len)
{
    struct sockaddr_in remoteInternet;
    int remoteLen = sizeof(struct sockaddr_in);
    int sock_fd;

    sock_fd = ::accept(sock, (sockaddr *)&remoteInternet, &remoteLen);

    if (from != NULL) *from = remoteInternet;
    if (len != NULL) *len = remoteLen;

    return(sock_fd);
}

